program _TajnyBMP_;
type
  tbmphlavicka = record
                   znacka    : array[0..1] of char; {Znacka BMP souboru}
                   velikost  : longint;             {Velikost BMP souboru}
                   reserved  : longint;             {Reserved}
                   velikosthl: longint;             {Velikost hlavicky}
                   velikostin: longint;             {Velikost informaci za hlavickou}
                   sirka     : longint;             {Sirka obrazku}
                   vyska     : longint;             {Vyska obrazku}
                   pocbitrov : word;                {Pocet bitovych rovin}
                   bits      : word;                {Pocet bitu na barvu}
                   komprimace: longint;             {Komprimace}
                   velobr    : longint;             {Velikost obrazku}
                   rx        : longint;             {Rastr v ose x}
                   ry        : longint;             {Rastr v ose y}
                   used, impo: longint;             {Pouzite a dulezite barvy}
                 end;

var
  BMPsoubor1,
  BMPsoubor2: file;
  hlavicka  : tbmphlavicka;
  i, j      : byte;
  tajnytxt  : string;
  paleta    : array[0..3] of byte;                  {Paleta barev; BGRI}
  precteno,
  zapsano   : word;
  buf       : array[1..2048] of char;

begin
  if paramcount < 3 then
  begin
    writeln('Pouziti programu: TAJNYBMP vstupni_soubor vystupni_soubor tajny_text');
    writeln('  - ulozi do BMP souboru tajny text');
    writeln('  - vstupni_soubor : - soubor, ze ktereho nacte obrazek');
    writeln('  - vystupni_soubor: - soubor, do ktereho se ulozi obrazek s tajnym textem');
    writeln('                     - pokud tento soubor existuje, bude prepsan!');
    writeln('  - funguje pouze s 1, 2, 4 a 8 bitovymi obrazky (text se uklada do palety)');
    writeln('  - delka textu: 1 bit : 2 znaky');
    writeln('                 2 bity: 4 znaky');
    writeln('                 4 bity: 16 znaku');
    writeln('                 8 bitu: 255 znaku');
    writeln('                 Zalezi jeste na poctu pouzitych barev!');
    writeln;
    exit;
  end;
  writeln('Filip Louzecky, program TajnyBMP');
  writeln;
  tajnytxt := '';
  for i := 3 to paramcount do
    if i <> paramcount then tajnytxt := tajnytxt + paramstr(i) + ' '
    else tajnytxt := tajnytxt + paramstr(i);
  writeln('INF: Zadany text: ', tajnytxt);
  for i := 1 to length(tajnytxt) do
    if not (ord(tajnytxt[i]) in [32..127]) then tajnytxt[i] := ' ';
  writeln('INF: Uprava zadaneho textu na tvar: ', tajnytxt);
  assign(BMPsoubor1, paramstr(1));
  {$i-}
  reset(BMPsoubor1, 1);
  {$i+}
  if ioresult <> 0 then
  begin
    writeln('ERR: Zadany soubor (', paramstr(1), ') je poskozeny nebo neexistuje.');
    halt;
  end;
  {$i-}
  blockread(BMPsoubor1, hlavicka, sizeof(hlavicka)); {Nacteni hlavicky}
  {$i+}
  if ioresult <> 0 then
  begin
    close(BMPsoubor1);
    writeln('ERR: Nelze precist zahlavi souboru (', paramstr(1), ').');
    halt;
  end;
  writeln('INF: Nacteno zahlavi souboru (', paramstr(1), ').');
  if (hlavicka.bits in [1, 2, 4, 8]) then
  begin
    if hlavicka.used <> 0 then j := hlavicka.used - 1
    else
    begin
      case hlavicka.bits of
        1: j := 1;
        2: j := 3;
        4: j := 15;
        8: j := 255;
      end;
    end;
  end
  else
  begin
    close(BMPsoubor1);
    writeln('ERR: Program dokaze zpracovat pouze 1, 2, 4 a 8 bitove obrazky (', paramstr(1), ').');
    halt;
  end;
  writeln('INF: Zjistena barevna hloubka (', hlavicka.bits, ') a pocet barev v palete (', j + 1, ').');
  if length(tajnytxt) > j + 1 then
  begin
    close(BMPsoubor1);
    writeln('ERR: Tajny text (', length(tajnytxt), ') je delsi nez paleta barev (', j + 1, ').');
    halt;
  end;
  assign(BMPsoubor2, paramstr(2));
  {$i-}
  rewrite(BMPsoubor2, 1);
  {$i+}
  if ioresult <> 0 then
  begin
    close(BMPsoubor1);
    writeln('ERR: Nelze otevrit soubor pro zapis (', paramstr(2), ').');
    halt;
  end;
  {$i-}
  blockwrite(BMPsoubor2, hlavicka, sizeof(hlavicka));
  {$i+}
  if ioresult <> 0 then
  begin
    close(BMPsoubor1);
    close(BMPsoubor2);
    writeln('ERR: Nelze zapsat zahlavi souboru (', paramstr(2), ').');
    halt;
  end;
  writeln('INF: Zapsano zahlavi souboru (', paramstr(2), ').');
  for i := 0 to j do
  begin
    {$i-}
    blockread(BMPsoubor1, paleta, sizeof(paleta));
    {$i+}
    if ioresult <> 0 then
    begin
      close(BMPsoubor1);
      close(BMPsoubor2);
      writeln('ERR: Chyba pri nacitani palety barev (', paramstr(1), ').');
      halt;
    end;
    if i + 1 <= length(tajnytxt) then paleta[3] := ord(tajnytxt[i+1]);
    {$i-}
    blockwrite(BMPsoubor2, paleta, sizeof(paleta));
    {$i+}
    if ioresult <> 0 then
    begin
      close(BMPsoubor1);
      close(BMPsoubor2);
      writeln('ERR: Chyba pri ukladani palety barev (', paramstr(2), ').');
      halt;
    end;
  end;
  writeln('INF: Nactena paleta (', paramstr(1), ') a zapsana paleta se zpravou (', paramstr(2), ').');
  repeat
    {$i-}
    blockread(BMPsoubor1, buf, sizeof(buf), precteno);
    blockwrite(BMPsoubor2, buf, precteno, zapsano);
    {$i+}
    if ioresult <> 0 then
    begin
      close(BMPsoubor1);
      close(BMPsoubor2);
      writeln('ERR: Chyba pri nacitani (', paramstr(1), ') nebo zapisovani (', paramstr(2), ') dat.');
    end;
  until (precteno = 0) or (precteno <> zapsano);
  writeln('INF: Zkopirovana obrazova data do souboru (', paramstr(2), ').');
  close(BMPsoubor1);
  close(BMPsoubor2);
  writeln('INF: Program uspesne zapsal tajnou zpravu do souboru (', paramstr(2),').');
  writeln;
end.